//
//  ClientSocket.m
//  ClientSocket
//
//  Created by fred yu on 8/4/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "ClientSocket.h"

#import <sys/socket.h>
#import <netinet/in.h>
#import <netdb.h>
#import <arpa/inet.h>

#import <SystemConfiguration/SCNetworkReachability.h>



@implementation ClientSocket


@synthesize getSeverArray;

static ClientSocket *sharedInst = nil;

#pragma mark Class Methods

+ (id)client
{
	@synchronized( self ) {
        if ( sharedInst == nil ) {
            
            sharedInst = [[self alloc] init];
        }
    }
	
    return sharedInst;
}

+ (id)clientWithHost:(NSString *)initHost port:(int)initPort andDelegate:(id)initDl
{
	@synchronized( self ) {
        if ( sharedInst == nil ) {
            
            sharedInst = [[self alloc] initWithHost:initHost port:initPort delegate:initDl];
        }
    }
	
    return sharedInst;
}

+ (id)clientConnectedTo:(NSString *)initHost onPort:(int)initPort withDelegate:(id)initDl
{
	@synchronized( self ) {
        if ( sharedInst == nil ) {
            
            sharedInst = [[self alloc] initWithHost:initHost port:initPort delegate:initDl];
        }
    }
	
    return sharedInst;
}


#pragma mark Instance Methods

- (id)init
{
	self = [super init];
	
	getSeverArray = [[NSMutableArray alloc] init];
	thisMsgLenth = 0;
	IsLastEnd = YES;
	
	return self;
}

- (id)initWithHost:(NSString *)initHost port:(int)initPort delegate:(id)initDl{
	
	if((self = [self init])) {
		if(![self setRemoteHost:initHost])
			return nil;
		if(![self setRemotePort:initPort])
			return nil;
		if(![self setDelegate:self])
			return nil;
	} else {
		return nil;
	}
	return self;
}

- (void)dataRelease{
	
	[getSeverArray release];
	getSeverArray = nil;
}

- (void)dealloc
{
	if(isConnected) {
		[self disconnect];
	}
	
	//	[self dataRelease];
	[super dealloc];
}

#pragma mark Connecting


- (SCCInit)connect{
	
	if(isConnected)
		return SCCInitError_Connected;
	if(!remoteHost)
		return SCCInitError_Host;
	if(remotePort < 1)
		return SCCInitError_Port;
	if(!delegate)
		return SCCInitError_Delegate;
	
	stayingConnected = YES;
	
	filedescriptor = -1;
	//创建一个指定协议和类型的CFSocket
	
	NSLog(@"ClientSocket::connect 创建一个指定协议和类型的CFSocket");
	
	mCFSocket = CFSocketCreate(kCFAllocatorDefault, PF_INET, SOCK_STREAM, IPPROTO_TCP, kCFSocketReadCallBack, NULL, NULL);
	CFSocketSetSocketFlags(mCFSocket, kCFSocketCloseOnInvalidate);
	
	if(mCFSocket) {
		
		//返回本地socket与CFSocketRef的连接，－1为socket连接失败
		filedescriptor = CFSocketGetNative(mCFSocket);
		
		//this code prevents the socket from existing after the server has crashed or been forced to close		
		int yes = 1;
		setsockopt(filedescriptor, SOL_SOCKET, SO_REUSEADDR, (void *)&yes, sizeof(yes));
		
		struct sockaddr_in addr4;
		memset(&addr4, 0, sizeof(addr4));
		addr4.sin_len = sizeof(addr4);
		addr4.sin_family = AF_INET;
		addr4.sin_port = htons(remotePort);
		//用在服务器端，监听所有的客户端连接
		//addr4.sin_addr.s_addr = htonl(INADDR_ANY);
		
		//IP地址转换，inet_pton处理ipv4、ipv6
		inet_pton(AF_INET, [remoteHost UTF8String], &addr4.sin_addr);
		//NSLog(@"data length: %d ,*******************",sizeof(addr4));
		NSData *address4 = [NSData dataWithBytes:&addr4 length:sizeof(addr4)];		
		//NSLog(@"ClientSocket::connect address is: %@",address4);
		
		//与远程socket建立连接

		int retVal = CFSocketConnectToAddress(mCFSocket, (CFDataRef)address4, [self connectionTimeout]);
		
		//printf("ClientSocket::connect 与远程socket建立连接 retVal is: %d",retVal);
		if(retVal == kCFSocketError)
			return SCCInitError_NoConnection;
		if(retVal == kCFSocketTimeout)			
			return SCCInitError_Timeout;
		if(retVal != kCFSocketSuccess)
			return SCCInitError_Unknown;
		
		CFRunLoopRef cfrl = CFRunLoopGetCurrent();
		source = CFSocketCreateRunLoopSource(kCFAllocatorDefault, mCFSocket, 0);
		CFRunLoopAddSource(cfrl, source, kCFRunLoopCommonModes);
		//		CFRelease(source);
		
		
	} else {
		return SCCInitError_NoSocket;
		
		
		printf("ClientSocket::connect SCCInitError_NoSocket");
	}
	
	//创建NSFileHandle
	
	
	
	if (nil != fileHandle) {
		[fileHandle release];
		fileHandle = nil;
	}
	fileHandle = [[NSFileHandle alloc] initWithFileDescriptor:filedescriptor
											   closeOnDealloc:NO];
	
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	if([self stayingConnected]) {
		
		//接收函数
		[nc addObserver:self
			   selector:@selector(dataReceivedNotification:)
				   name:NSFileHandleReadCompletionNotification
				 object:nil];
		
		[fileHandle readInBackgroundAndNotify];
	} 
	else {
		[nc addObserver:self
			   selector:@selector(dataReceivedNotification:)
		 //			   selector:@selector(disconnect)
				   name:NSFileHandleReadToEndOfFileCompletionNotification
				 object:nil];
		[fileHandle readToEndOfFileInBackgroundAndNotify];
	}
	
	isConnected = YES;
	
	
	
	return SCCInitOK;
}

- (void)disconnect{
	
	NSLog(@"网络链接断开");
	
	
	if (nil == fileHandle) {
		return;
	}
	[self NSFileHandlecloseFile];
	//	[self unscheduleFromRunLoop];
	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	CFSocketInvalidate(mCFSocket);
	CFRelease(mCFSocket);
	mCFSocket = nil;
	
	if(fileHandle != nil)
	{
		[fileHandle release];
		fileHandle = nil;
	}
	isConnected = NO;
	
}

- (void)unscheduleFromRunLoop
{
	if( source == NULL )
		return;
	
	if( !CFRunLoopSourceIsValid(source) )
		return;
	
	CFRunLoopSourceInvalidate(source);
	CFRelease(source);
	
	source = NULL;
}

- (void) NSFileHandlecloseFile{
	
	[fileHandle closeFile];
	CFSocketDisableCallBacks(mCFSocket, kCFSocketAcceptCallBack);
}

#pragma mark Sending and Receiving

- (BOOL)sendData:(NSData *)data{
	

	
	@try {
		[fileHandle writeData:data];
		NSLog(@"ClientSocket::sendData data is: %@", data);
		//		NSData* a = [fileHandle availableData];
		NSLog(@"send data：%d",[data length]);
    }
    @catch (NSException *exception) {
		//[self processDataSendingError:exception forConnection:con]; //Not used in this version. Perhaps used and documented in future versions.
		//return NO;
		
		NSLog(@"ClientSocket::sendData exception");
    }
	
	return YES;
}

- (BOOL)sendString:(NSString *)string{
	
	//	NSLog(@"\n********************************\n%@",string);
	//return [self sendData:[string dataUsingEncoding:[self defaultStringEncoding]]];
	
	//[aStr dataUsingEncoding: NSASCIIStringEncoding];
	
	
	return [self sendData:[string dataUsingEncoding:NSASCIIStringEncoding]];
}

- (BOOL)sendString:(NSString *)string withEncoding:(NSStringEncoding)encoding{
	
	return [self sendData:[string dataUsingEncoding:encoding]];
}

- (void)dataReceivedNotification:(NSNotification *)notification{
	
	
	NSData *data = [NSData dataWithData:[[notification userInfo] objectForKey:NSFileHandleNotificationDataItem]];
	printf("\nLength:%d\n",[data length]);
	NSLog(@"ClientSocket::dataReceivedNotification data is: %@", data);
	
	//返回值为0，不再连接sever
	if([data length] <= 0) {
		
		[self disconnect];
		return;
	}	
	
	if([data length] > 0){
		
		NSString* aStr;
		aStr = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
		//NSLog(@"ClientSocket::dataReceivedNotification dataString is: %@",aStr);
		
		[getSeverArray addObject:data];
		//		NSLog(@"getSeverArray count:%d",[getSeverArray count]);
	}
	
	//没有离开游戏继续监听
	if (nil != fileHandle) {
		[fileHandle readInBackgroundAndNotify];
	}else {
		[self disconnect];
	}
}

//获取后的信息处理
- (void) processData:(NSData*)data{
	NSLog(@"ClientSocket::processData data is: %@", data);
	if (thisMsgLenth == 0) {
		
		[data getBytes:&thisMsgLenth range:NSMakeRange(0, 2)];
		thisMsgLenth = ntohs(thisMsgLenth);
		
		//消息接收完全
		int newLenth = [[getSeverArray objectAtIndex:0] length];
		if (thisMsgLenth != newLenth){
			IsLastEnd = NO;
			if (thisMsgLenth < newLenth) {
				
				NSMutableData* nextData = [getSeverArray objectAtIndex:0];
				NSData* popData = [nextData subdataWithRange:NSMakeRange(0, thisMsgLenth)];	
				NSData* aData = [nextData subdataWithRange:NSMakeRange(thisMsgLenth, [nextData length]-thisMsgLenth)];	
				
				[getSeverArray insertObject:popData atIndex:1];
				[getSeverArray insertObject:aData atIndex:2];
				[getSeverArray removeObjectAtIndex:0];
			}
			if (thisMsgLenth > newLenth) {
				
				NSMutableData* newData = [NSMutableData dataWithData:[getSeverArray objectAtIndex:0]];
				if ([getSeverArray count]>=2) {
					NSData* nextData = [NSData dataWithData:[getSeverArray objectAtIndex:1]];
					[newData appendData:nextData];
					[getSeverArray replaceObjectAtIndex:0 withObject:newData];
					[getSeverArray removeObjectAtIndex:1];
					thisMsgLenth = 0;
				}
			}
			newLenth = [[getSeverArray objectAtIndex:0] length];
		}
		
		if (thisMsgLenth == newLenth) {
			thisMsgLenth = 0;
			IsLastEnd = YES;
		}
	}
	
	//	NSLog(@"\nArray Count:%d\n",[getSeverArray count]);
}

#pragma mark Other Methods

- (void)setIsConnected:(BOOL)val{
	
	isConnected = val;
}

- (BOOL)setNewHost:(NSString *)newHost andPort:(int)newPort{
	
	if(isConnected)
		return NO;
	if(![self setRemoteHost:newHost])
		return NO;
	if(![self setRemotePort:newPort])
		return NO;
	return YES;
}

- (BOOL)isConnected {
	
	return isConnected;
}

#pragma mark Accessor Methods

- (id)delegate{
	
	return delegate;
}

- (BOOL)setDelegate:(id)newDl{
	
	if(isConnected)
		return NO;
	newDl = [newDl retain];
	[delegate release];
	delegate = newDl;
	return YES;
}

- (NSString *)remoteHost{
	
	return remoteHost;
}

- (NSString *)remoteHostName{
	
	struct in_addr IPaddr;
	struct hostent *host;
	inet_pton(AF_INET, [remoteHost UTF8String], &IPaddr);
	host = gethostbyaddr((char *) &IPaddr, sizeof(IPaddr),AF_INET);
	return [NSString stringWithUTF8String:(host->h_name)];
}

- (BOOL)setRemoteHost:(NSString *)newHost{
	
	if(isConnected)
		return NO;
	newHost = [newHost retain];
	if(!newHost)
		return NO;
	
	struct in_addr IPaddr;
	struct hostent *host;
	int IPok = inet_pton(AF_INET, [newHost UTF8String], &IPaddr);
	if(!IPok) {
		host = gethostbyname([newHost UTF8String]);
		if(host) {
			[newHost release];
			char ipchar[16];
			//转换字符串到网络地址,转换为in_addr的结构体
			inet_ntop(AF_INET,host->h_addr_list[0], ipchar, sizeof(ipchar));
			newHost = [NSString stringWithUTF8String:ipchar];
			
			NSLog(@"newHost:%@",newHost);
			
		} else
			return NO;
	}
	remoteHost = newHost;
	return YES;
}


- (int)remotePort{
	
	return remotePort;
}

- (BOOL)setRemotePort:(int)newPort
{
	if(isConnected)
		return NO;
	remotePort = newPort;
	return YES;
}

- (BOOL)stayingConnected{
	
	return stayingConnected;
}

- (void)setStayingConnected:(BOOL)flag{
	
	stayingConnected = flag;
}

- (int)connectionTimeout
{
	if(connectionTimeout < 3)
		return SCCDefaultConnectionTimeout;
	return connectionTimeout;
}

- (void)setConnectionTimeout:(int)newTimeout
{
	connectionTimeout = newTimeout;
}

- (NSStringEncoding)defaultStringEncoding
{
	return defaultStringEncoding;
}

- (void)setDefaultStringEncoding:(NSStringEncoding)encoding
{
	defaultStringEncoding = encoding;
}

@end

